//bump.cpp

#include "bump.h"

//-------------------------------------------------------------------------------------

Bump::Bump() {
	size=128;
	freq=1;
	return;
};

Bump::~Bump() {};

//--------------------------------------------------------------------

int Bump::load(const char *picname, float mag, float off) {
	FILE * file;
	int x,y, addr, yaddr, red,green,blue, sizex, sizey;
	float height;

	file= fopen(picname, "rb");
	if (file == 0) return 1;

	fseek(file, 18, SEEK_SET);
	sizex=fgetc(file)+fgetc(file)*256;
	fseek(file, 22, SEEK_SET);
	sizey=fgetc(file)+fgetc(file)*256;
	if (sizex*sizey>65536) return 1;
	size=sizex;

	fseek(file, 54, SEEK_SET);
	for     (y = 0; y <size; y++) {
		yaddr=(size-y-1)*size;
		for (x = 0; x <size; x++) {
			addr=x+yaddr;
			blue =fgetc(file);
			green=fgetc(file);
			red  =fgetc(file);
			height=(green/256.)*mag*255+off; // use green channel
			height=mini(maxi(height,0),255);
			//(red+green+blue)/(3*256.);
			if (addr<65536) bumparray[addr]=height;
		}
	}
	fclose(file);
	return 0;
}

//---------------------------------------------------------------------------

int Bump::make(int mode, int insize, float mag, float off) {
	int x,y, addr;
	float height, s2, dist;
	size=insize;
	s2=size*.5;
	for    (y = 0; y <size; y++) {
		for (x = 0; x <size; x++) {
			addr=(size-x-1)+(size-y-1)*size;
			switch(mode) {
			case 0:	height=off; break; // flat
			case 1: // playground 
				if (y<20) { // big v ramp
					if (x<111) height = cosf((((131.75/128.)*x)/size)*pi2-.3)*2.4-1.4;
					else height=0;
				}
				else if ((x<20)&&(y<84)&&(y>=20)){ // ramp up
					height=cos_interp(1,0,((float)(y-20)/64));
				}
				else if (y<28) { // wall
					if (x<111) height=1; else height=0;
				}
				else if (y<36) { // curve
					if (x<111) height=-sinf((y-28)*.225)+1; else height=0;
				}
				else if ((y>90)&&(x>20)&&(x<108)) 
					if (x<64) height=(cosf((y-90)*.18)-1)*.4; // trough
					else      height=(cosf((y-90)*.18)-1)*.4 * (-cosf((x-21)*.14612)+1)*.04; // slight well
				else height=0;

				dist=sqrt((x-s2+5)*(x-s2+5)+(y-s2+2)*(y-s2+2))/s2;// big hole
				if (dist<.3.33) {
					dist*=3;
					height=(-cosf(dist*pi1)-1)*.6;
				}
				if ((x>90)&&(x<110)&&(y>55)&&(y<65)) { // small v ramp
					height=(cosf((x-90.)/20.*pi2)-1)*.2;
				}
				if (((127-x)+(127-y))<10) { // pull up corners
					height=(10-((127-x)+(127-y)))*.04;
				}
				if ((x+(127-y))<10) {
					height=(10-(x+(127-y)))*.04;
				}
				break;
			}
			if (addr<65536) bumparray[addr]=height*mag+off;
		}
	}
	return 0;
}

//---------------------------------------------------------------------------

float Bump::get(float x, float y) { // 0-1, 0-1 -> 0-1
	int ix, iy, addr;
	float v=0, v0=0, xi,yi;

	while (x<0) x+=1;
	while (y<0) y+=1;
	while (x>1) x-=1;
	while (y>1) y-=1;
	if (x>.999) x=.999;
	if (y>.999) y=.999;
	ix=x*size; 
	iy=y*size;
	xi=x*size-ix;
	yi=y*size-iy;
	//ix%=size;
	//iy%=size;
	//if (ix>=size) ix=size-1;
	//if (iy>=size) iy=size-1;
	//if (xi>=1) xi=0;
	//if (yi>=1) yi=0;

	addr=ix+iy*size;
	if (addr<65536) v0=bumparray[addr];
	v=v0;

	addr=((ix+1)%size)+iy         *size;
	if (addr<65536) v+=(bumparray[addr]-v0)*xi;

	addr=  ix+       ((iy+1)%size)*size;
	if (addr<65536) v+=(bumparray[addr]-v0)*yi;
	return v;
}

//--------------------------------------------------------------------

