//  text.cpp

#include "text.h"
#include "hardware.h"

//----------------------------------------------------------------------------

Text::Text() {
	autoset=1;
	size=12; 
	reset();
	s[0]=0;
};

Text::~Text() {};

Texture *Text::tex=0;

int   Text::charInX=16;
int   Text::charInY=8;
float Text::txoffy=0;

//---------------------------------------------------------------------------

void Text::reset(void) {
	x=20; y=20; xinc=.9; yinc=0;
	r=g=b=a=1; inten=1;
	alphatest=1; alphafunc=.1;
	blend=0; blendFunc0=2; blendFunc1=3; depthtest=0; depthwrite=0; italic=3;
	duration=3; endTime=0; 
	bend=0;   bendspeedx=1;  bendspeedy=1;  bendamplx=1;  bendamply=1;
	bloom=0; bloomspeedx=1; bloomspeedy=1; bloomamplx=1; bloomamply=1;
}

//---------------------------------------------------------------------------

void Text::hold(char *ins) {
	strcpy(s,ins); 
	endTime=Hw::get()->time.realTime+duration;	return;
}

//---------------------------------------------------------------------------

void Text::hold(char *format, float value) {
	sprintf(s,format,value);
	endTime=Hw::get()->time.realTime+duration;	return;
}

//---------------------------------------------------------------------------

void Text::hold(char *format, int value) {
	sprintf(s,format,value);
	endTime=Hw::get()->time.realTime+duration;	return;
}

//---------------------------------------------------------------------------

void Text::hold(char *format, char *value) {
	sprintf(s,format,value);
	endTime=Hw::get()->time.realTime+duration;	return;
}

//---------------------------------------------------------------------------

void Text::print_hold(void) {
	if (endTime) {
		if (Hw::get()->time.realTime<endTime) {
			print(s, x, y, size);
		} else {
			endTime=0;
			s[0]=0;
			if ((Hw::get()->screen.letterbox)||(!Hw::get()->pref.blank)) Hw::get()->screen.needblank=2;
		}
	}
}

//---------------------------------------------------------------------------

void Text::print(char *ins) {
	print(ins, x,y, size);
}

//--------------------------------------------------------------------------------

void Text::print(char *ins, float inx, float iny, float insize) {
	unsigned char c;
	unsigned int i;
	float time=Hw::get()->time.time;

	float dx,dy, fx,fy, tx,ty, tx2,ty2, xsize,ysize, bendx=0,bendy=0,
		   txsize=1./charInX,
			tysize=1./charInY;

	strcpy(s,ins);
	dx=x=inx;
	dy=y=iny;
	size=insize;

	if (autoset) set();

	glPushAttrib(GL_POLYGON_BIT);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	for (i=0; i<strlen(s); i++) {
		if (bend) {
			bendx=sinf(time*bendspeedx*2  +i)*size*bendamplx*.125;
			bendy=sinf(time*bendspeedy*2.5+i)*size*bendamply*.125;
		}
		if (bloom) {
		xsize=size*.5*(sinf(time*bloomspeedx*3  +i)*bloomamplx*.125+1);
		ysize=size   *(sinf(time*bloomspeedy*3.5+i)*bloomamply*.125+1);
		} else {
		xsize=size*.5;
		ysize=size;
		}
		c=s[i]-32;
		tx=  (c%charInX)*txsize;
		ty=1-(c/charInX)*tysize-tysize+txoffy;
		tx2=tx+txsize;
		ty2=ty+tysize;
		glBegin(GL_QUADS);
			glTexCoord2f(tx ,ty ); fx=dx-xsize+bendx;        fy=dy-ysize-bendy; glVertex2f(fx,fy);
			glTexCoord2f(tx2,ty ); fx=dx+xsize-bendx;        fy=dy-ysize+bendy; glVertex2f(fx,fy);
			glTexCoord2f(tx2,ty2); fx=dx+xsize+bendx+italic; fy=dy+ysize-bendy; glVertex2f(fx,fy);
			glTexCoord2f(tx ,ty2); fx=dx-xsize-bendx+italic; fy=dy+ysize+bendy; glVertex2f(fx,fy);
			glEnd();
		dx+=size*xinc;
	}
	if (yinc) y+=size*yinc;
	glPopAttrib();

	if (autoset) unset();
	Hw::get()->pref.ntris+=strlen(s)*2;
	return;
	}

//--------------------------------------------------------------------------------

void Text::hold_multi(int value, char *s0,char *s1,char *s2,char *s3,char *s4,char *s5,
							            char *s6,char *s7,char *s8,char *s9,char *s10){
	switch(value){
		case 0: hold(s0); break;
		case 1: hold(s1); break;
		case 2: hold(s2); break;
		case 3: hold(s3); break;
		case 4: hold(s4); break;
		case 5: hold(s5); break;
		case 6: hold(s6); break;
		case 7: hold(s7); break;
		case 8: hold(s8); break;
		case 9: hold(s9); break;
		case 10: hold(s10); break;
	}
}

//--------------------------------------------------------------------------------

void Text::set(void){
	if (alphatest) {
		glAlphaFunc(GL_GREATER,alphafunc);
		glEnable(GL_ALPHA_TEST);
	}
	if (blend) {
		glEnable(GL_BLEND);
		set_blend(blendFunc0,blendFunc1);
	}
	glColor4f(r*inten,g*inten,b*inten,a*inten);
	if (depthtest)   glEnable(GL_DEPTH_TEST);
	if (!depthwrite) glDepthMask(GL_FALSE);
	glEnable(GL_TEXTURE_2D);
	if (tex!=0) tex->bind();
}

//--------------------------------------------------------------------------------

void Text::unset(void){
	if (alphatest) glDisable(GL_ALPHA_TEST);
	if (blend)     glDisable(GL_BLEND);
	if (depthtest) glDisable(GL_DEPTH_TEST);
	if (!depthwrite) glDepthMask(GL_TRUE);
	glDisable(GL_TEXTURE_2D);
}

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------

Scroller::Scroller() {
	buflen=200;
	reset();
};

Scroller::~Scroller() {};

//--------------------------------------------------------------------------------

int Scroller::msg(const char *ins){
	if ((strlen(sbuff)+strlen(ins))>buflen-2) {// too long?
		log_message("Too long scroll message.");
		return 1;
	}
	if (strlen(sbuff)!=0) strcat(sbuff,"   "); // insert space between messages
	strcat(sbuff,ins); // add message to buffer
	return 0;
};

//--------------------------------------------------------------------------------

void Scroller::print(void){
	float xi;

	scroll+=scrolli*Hw::get()->time.dt*Hw::get()->pref.scrollSpeed; // finely scroll character
	if (scroll<0) { // time to shift character?
		scroll+=1;
		for (unsigned int i=0; i<99; i++) abuff[i]=abuff[i+1]; // shift screen buffer to left
		if (sbuff[0]!=0) abuff[99]=sbuff[0]; // load a new character
		else             abuff[99]=' ';
		abuff[100]=0;
		for(i=0; i<buflen; i++) sbuff[i]=sbuff[i+1]; // shift msg buffer
		sbuff[buflen]=0;
	}
	xi=-50+scroll*xinc*size;
	Text::print(abuff,xi,y,size);
};

//--------------------------------------------------------------------------------

void Scroller::reset(void){
	for(int i=0; i<100; i++) abuff[i]=' ';
	abuff[100]=0;
	sbuff[0]=0;
	scroll=0;
	scrolli=-16;
	size=14;
	y=40;
}

//--------------------------------------------------------------------------------

