//       visual attribute class w/
//          20+ render params
//          smart state setter/unsetter (non-sorting)

#include "vis.h"
#include "hardware.h"

//-------------------------------------------------------------------
//-------------------------------------------------------------------
//-------------------------------------------------------------------

Vis::Vis(){
	alphatest=0; alphafunc=.01;
	cull=0; blend=0; blendFunc0=1; blendFunc1=1;
	depthtest=1; depthwrite=1;
	fog=1; lights=1; textures=1; polyo=0;// 0-off 1-on
	r=g=b=a=1;
	texmagx=texmagy=1;
	texoffx=texoffy=0;
	tex=0;
	image=0;
};
Vis::~Vis(){};

int Vis::galphateston=0;
int Vis::gblendon    =0;
int Vis::gcullon    =0;
int Vis::gdepthteston=0;
int Vis::gdepthwriteon=1;
int Vis::gfogon      =0;
int Vis::glightson   =0;
int Vis::gtextureson =0;

//------------------------------------------------------------------------

void Vis::set(void) { // state manager for no redundant calls
	switch(alphatest) {
		case 0: if (galphateston) {glDisable(GL_ALPHA_TEST);galphateston=0;}
			break;
		case 1: if (!galphateston) {glEnable(GL_ALPHA_TEST);galphateston=1;}
			glAlphaFunc(GL_GREATER,alphafunc);
			break;
	}
	switch(blend) {
		case 0: if (gblendon) {glDisable(GL_BLEND);gblendon=0;}
			break;
		case 1: if (!gblendon) {glEnable(GL_BLEND);gblendon=1;}
			if (Hw::get()->pref.colorMode!=5) set_blend(blendFunc0,blendFunc1);
			else                              set_blend(2,1);
			break;
	}

	switch(cull) {
		case 0: if ( gcullon) {glDisable(GL_CULL_FACE);gcullon=0;}
			break;
		case 1: if (!gcullon) { glEnable(GL_CULL_FACE);gcullon=1;}
			break;
	}

	zColor4f(r,g,b,a);

	switch(depthtest) {
		case 0: if (gdepthteston) {glDisable(GL_DEPTH_TEST);gdepthteston=0;}
			break;
		case 1: if (!gdepthteston) {glEnable(GL_DEPTH_TEST);gdepthteston=1;}
			break;
	}

	switch(depthwrite) {
		case 0: if (gdepthwriteon) {glDepthMask(GL_FALSE);gdepthwriteon=0;}
			break;
		case 1: if (!gdepthwriteon) {glDepthMask(GL_TRUE);gdepthwriteon=1;}
			break;
	}

	if ((fog==0)&&( gfogon)) {glDisable(GL_FOG);gfogon=0;}
	if ((fog!=0)&&(Hw::get()->pref.fog)) {
		if (!gfogon) {
			glEnable(GL_FOG);
			gfogon=1;
		}
		glFogf(GL_FOG_START,Hw::get()->pref.b1*fog+Hw::get()->pref.brace);
		glFogf(GL_FOG_END,  Hw::get()->pref.b2*fog+Hw::get()->pref.brace);
	}
	switch(lights) {
		case 0: if (glightson) {glDisable(GL_LIGHTING); glightson=0;}
			break;
		case 1: if ((!glightson)&&(Hw::get()->pref.lights)) {glEnable(GL_LIGHTING);glightson=1;}
			break;
	}
	switch(textures) {
		case 0: if (gtextureson) {glDisable(GL_TEXTURE_2D); gtextureson=0;}
			break;
		case 1: if ((!gtextureson)&&(Hw::get()->pref.textures)) {glEnable(GL_TEXTURE_2D);gtextureson=1;}
			if (tex) Texture::textures[image]->bind();
			break;
	}
}

//-------------------------------------------------------------------------------

void Vis::reset(void) {
	galphateston =0;
	gblendon     =0;
	gcullon      =0;
	gdepthteston =0;
	gdepthwriteon=1;
	gfogon       =0;
	glightson    =0;
	gtextureson  =0;
}

//-------------------------------------------------------------------------------

void Vis::unset(void) {
	if (galphateston)  {glDisable(GL_ALPHA_TEST);galphateston=0;}
	if (gblendon)      {glDisable(GL_BLEND);     gblendon    =0;}
	if (gcullon)       {glDisable(GL_CULL_FACE); gcullon     =0;}
	if (gdepthteston)  {glDisable(GL_DEPTH_TEST);gdepthteston=0;}
	if (!gdepthwriteon){glDepthMask(GL_TRUE);   gdepthwriteon=1;}
	if (gfogon)	       {glDisable(GL_FOG);       gfogon      =0;}
	if (glightson)	    {glDisable(GL_LIGHTING);  glightson   =0;}
	if (gtextureson)	 {glDisable(GL_TEXTURE_2D);gtextureson =0;}
}

//------------------------------------------------------------------------

void Vis::set_color(float inr,float ing,float inb) {
	r=inr;
	g=ing;
	b=inb;
}

//-------------------------------------------------------------------

void Vis::set_color(float inr,float ing,float inb, float ina) {
	r=inr;
	g=ing;
	b=inb;
	a=ina;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
//-------------------------------------------------------------------

